@@ -129,10 +129,10 @@ class Assignment(object):
         if self.value is None:
             raise Exception("can't translate unknown value")
         if self.sentence.type == BOOL:
-            out = self.sentence.original if self.value.same_as(TRUE) else \
-                AUnary.make('¬', self.sentence.original)
+            out = self.sentence if self.value.same_as(TRUE) else \
+                AUnary.make('¬', self.sentence)
         else:
-            out = AComparison.make('=', [self.sentence.original, self.value])
+            out = AComparison.make('=', [self.sentence, self.value])
         return out
 
     def negate(self):
