@@ -249,7 +249,7 @@ AUnary.translate1 = translate1
 # Class AAggregate  #######################################################
 
 def translate1(self):
-    assert self.quantifier_is_expanded, "Cannot expand {self.code}"
+    assert self.quantifier_is_expanded, f"Cannot expand {self.code}"
     return Sum([f.translate() for f in self.sub_exprs])
 AAggregate.translate1 = translate1
 
